#!/bin/sh
#
#  This script does the following:
#  	(1) Downloads the MADIS data of interest, 
#	(2) Processes the data into a format usable by MET's ascii2nc, and
#	(3) Runs ascii2nc to create netcdf files for use in MET's point_stat program.
#
#  Author: Jonathan Case, NASA SPoRT Center/ENSCO, Inc., Huntsville, AL
#  Modification history:
#	-Initial development   (2/20/2009)
#	-First working version (3/4/2009) -- Initially designed to work on ONLY surface observations.
#
#  Requirements:
#	(1) Requires the accompanying shell scripts "ascii2nc_reformat.sh" and "dayadd" in the user's $PATH
# 	    or top level directory.
#	(2) Requires that the MADIS API be installed.
#	(2) Also requires the following MADIS files in which dates are substituted:
#		-ftp.par1.txt.sed
#		-api.par1.txt.sed
#		-sfcdump.par1.txt.land.sed
#		-sfcdump.par1.txt.maritime.sed
#
#  Notes:
#	(1) All surface and mesonet land reports are assigned a message_type of "ADPSFC", while
#           maritime observations are assigned the message_type "SFCSHP" for use in MET. 
#	(2) TOPDIR is defined as the user's top-level login directory in .bashrc
#
# Source the user's login file
. ~/.bashrc

# Set flags for (1) downloading MADIS data, 
#		(2) processing MADIS data with the MADIS API into a format usable by MET's ascii2nc, and
# 		(3) running ascii2nc to prepare the data for MET's point_stat program.

dodownload=n
doprocess=y
doascii2nc=y  # Note: "doprocess" must be activated to run "doascii2nc"

# list of MADIS-defined variables
varlist="T TD U V DD FF SLP"
#varlist=P
yyyymmdd_start="20080603"
yyyymmdd_end="20080901"

MADIS_TOP=${TOPDIR}/MADIS
MADIS_DATA=/raid1/casejl/DATA/MADIS
MADIS_WORK=${MADIS_DATA}/API_WORK
MADIS_OUT=${MADIS_DATA}/OUTPUT

MET_WORK=/raid1/casejl/MET/ASCII2NC
MET_OUT=${MET_WORK}/OUTPUT

# Download MADIS data if flag is turned on.
# NOTE: MADIS download requires a MADIS id and password.  

if [ $dodownload = "y" ] ; then

  cd $MADIS_WORK

# First download all netcdf MADIS data, as defined in the 
# Y/N flags in ftp.par1.txt.sed

  sed 's;YYYYMMDD_START;'${yyyymmdd_start}';g' ${MADIS_TOP}/ftp.par1.txt.sed |\
  sed 's;YYYYMMDD_END;'${yyyymmdd_end}';g' > ${MADIS_WORK}/ftp.par1.txt

  ln -fs ${MADIS_TOP}/get_MADIS_Data_unix.pl ./
./get_MADIS_Data_unix.pl

fi ## MADIS download section

# Next, run the MADIS API to extract variables of interest over the 
# same date/time interval, if the flag is turned on.

if [ $doprocess = "y" ] ; then

  cd ${MADIS_WORK}

  yyyymmdd=${yyyymmdd_start}
  while [[ ${yyyymmdd} -le ${yyyymmdd_end} ]] ; do

    cc=`echo $yyyymmdd | cut -c1-2`
    yymmdd=`echo $yyyymmdd | cut -c3-8`

    sed 's;YYYYMMDD_START;'${yyyymmdd}';g' ${MADIS_TOP}/api.par1.txt.sed |\
    sed 's;YYYYMMDD_END;'${yyyymmdd}';g' > ${MADIS_WORK}/api.par1.txt

    firstvar=y
    for var in ${varlist} ; do
# Separate out the land vs. maritime obs so that MET's point_stat can
# deliminate between the 2 different obs types.
    for type in land maritime ; do

      cd ${MADIS_WORK}

      echo "Processing variable $var for date $yyyymmdd"
      sed 's;VARNAME_SED;'${var}';g' ${MADIS_TOP}/sfcdump.par1.txt.${type}.sed > ${MADIS_WORK}/sfcdump.par1.txt

      ln -fs ${MADIS_TOP}/run_MADIS_API_unix.pl ./
      ./run_MADIS_API_unix.pl
      head -5 sfcdump.txt > ${MADIS_OUT}/${type}_${yyyymmdd}_${var}.txt
# Exclude APRSWXNET observations based on personal experience of their questionable quality.
      grep "V-${var}" sfcdump.txt | grep -v APRSWXNET >> ${MADIS_OUT}/${type}_${yyyymmdd}_${var}.txt
      rm -f sfcdump.txt

# Re-format to that required for MET's ascii2nc program, and
# combine all variables into a new single daily file.

      infname="${MADIS_OUT}/${type}_${yyyymmdd}_${var}.txt"
      outfname="${MADIS_OUT}/sfcdump_${yyyymmdd}.txt"
      
      ${TOPDIR}/ascii2nc_reformat.sh ${infname} ${type} > temp.txt

      if [ $firstvar = "y" ] ; then
        mv -f temp.txt daily.txt
        firstvar=n
      else
        cat temp.txt >> daily.txt
      fi
      rm -f temp.txt

    done ## type loop
    done ## var loop

# Sort daily.txt file by valid time (3rd column), outputting unique values,
# thereby discarding any duplicate observations.

    cat daily.txt | sort -k3,3 | uniq > ${outfname}
    rm -f daily.txt

# Run the MET ascii2nc program on the merged daily file created by
# the data processing section above.

    if [ $doascii2nc = "y" ] ; then
      cd ${MET_WORK}
      ncoutfname=${MET_OUT}/sfcobs_${yyyymmdd}.nc 
      echo "Running ascii2nc on ${outfname}"
      ascii2nc ${outfname} ${ncoutfname} -v 2
    fi

    echo "g-zipping MADIS intermediate text files."
    gzip -f ${MADIS_OUT}/*${yyyymmdd}*.txt
   
    yymmdd=`dayadd 1 $yymmdd`
    yyyymmdd=${cc}${yymmdd}

  done ## yyyymmdd loop

fi ## MADIS API data processing section.



exit
